// KLChgExec.cpp : Defines the entry point for the application.

#include "stdafx.h"
#include "KLChgExec.h"
#include "RegNotifyUtil.h"
#include "IniUtil.h"

/* Defines */
#define APP_CLASS_NAME			_T("KLChgExec Application Class")
#define APP_TITLE_NAME			_T("KLChgExec Application")
#define CONFIGFILE_NAME			_T("KLChgExec.ini")
#define APP_SECTION				_T("KLCHGEXEC")

#define APP_KEYLOCK_ON_APP		_T("KEYLOCK_ON_APP")
#define APP_KEYLOCK_ON_ARGS		_T("KEYLOCK_ON_ARGS")
#define APP_KEYLOCK_OFF_APP		_T("KEYLOCK_OFF_APP")
#define APP_KEYLOCK_OFF_ARGS	_T("KEYLOCK_OFF_ARGS")

#define PHONESTATUS_REG_KEY		_T("Software\\Sharp\\PhoneStatus")
#define KEYLOCK_REG_VALUE_NAME	_T("Status0")

#define WM_USER_APPEND_REQUEST	(WM_USER + 1)
#define WM_USER_STATUS0_CHANGE	(WM_USER + 2)


/* Typedefs */
typedef struct {
	LPTSTR	pKeyLockOnApp;
	LPTSTR	pKeyLockOnArgs;
	LPTSTR	pKeyLockOffApp;
	LPTSTR	pKeyLockOffArgs;
} CONFIGDATA;


/* Global Variables */ 
HINSTANCE	g_hInst = NULL;

/* ProtoTypes */
ATOM				MyRegisterClass	(HINSTANCE, LPTSTR);
BOOL				InitInstance	(HINSTANCE, int);
LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);


static
LPWSTR
LoadIniFileToBuffer(
	LPTSTR	pFileName
)
{
	HANDLE	hFile			= INVALID_HANDLE_VALUE;
	LPSTR	pAnsiFileBuf	= NULL;
	LPWSTR	pFileBuf		= NULL;
	int		iBufSize		= 0;

	hFile = CreateFile(pFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {

		BY_HANDLE_FILE_INFORMATION	fileInfo;

		memset(&fileInfo, 0, sizeof(fileInfo));

		if (GetFileInformationByHandle(hFile, &fileInfo) && fileInfo.nFileSizeLow > 0) {
			pAnsiFileBuf = (LPSTR) malloc(fileInfo.nFileSizeLow);
			if (pAnsiFileBuf != NULL) {
				DWORD	dwReaded = 0;

				if (ReadFile(hFile, pAnsiFileBuf, fileInfo.nFileSizeLow, &dwReaded, NULL)) {
					iBufSize = MultiByteToWideChar(CP_ACP, 0, pAnsiFileBuf, fileInfo.nFileSizeLow, NULL, 0);

					if (iBufSize > 0) {
						pFileBuf = (LPWSTR) malloc((iBufSize + 1)*sizeof(WCHAR));
						if (pFileBuf != NULL) {
							MultiByteToWideChar(CP_ACP, 0, pAnsiFileBuf, fileInfo.nFileSizeLow, pFileBuf, iBufSize + 1);
						}
					}

				}

				free(pAnsiFileBuf);
				pAnsiFileBuf = NULL;
			}
		}

		CloseHandle(hFile);
		hFile = NULL;
	}

	return pFileBuf;
}

static
void
DiscardIniBuffer(
	LPWSTR	pFileBuf
)
{
	if (pFileBuf != NULL) {
		free(pFileBuf);
	}
}

static
BOOL
GetIniPath(
	LPTSTR		pPath
)
{
	BOOL	bRet;
	TCHAR	szPath[MAX_PATH + 1];
	LPTSTR	p = NULL;

	bRet = FALSE;

	if (pPath == NULL) {
		return FALSE;
	}

	memset(szPath, 0, sizeof(szPath));
	GetModuleFileName(NULL, szPath, sizeof(szPath)/sizeof(szPath[0]));

	p = _tcsrchr(szPath, _T('\\'));
	if (p != NULL) {
		*(p+1) = _T('\0');

		_tcscpy(pPath, szPath);
		_tcscat(pPath, CONFIGFILE_NAME);

		bRet = TRUE;
	}

	return bRet;
}

static
BOOL
LoadConfigData(
	LPTSTR	pFilePath,
	CONFIGDATA* pConfigData
)
{
	BOOL	bRet;
	LPWSTR	pIniBuf = NULL;

	if (pFilePath == NULL || pConfigData == NULL) {
		return FALSE;
	}

	bRet = FALSE;	// default return value

	pIniBuf = LoadIniFileToBuffer(pFilePath);
	if (pIniBuf != NULL) {
		DWORD	dwSize = 0;
		LPTSTR	pKeyLockOnApp		= NULL;
		LPTSTR	pKeyLockOnArgs	= NULL;
		LPTSTR	pKeyLockOffApp		= NULL;
		LPTSTR	pKeyLockOffArgs		= NULL;

		dwSize = 0;
		if (GetIniString(APP_SECTION, APP_KEYLOCK_ON_APP, NULL, &dwSize, pIniBuf)) {
			pKeyLockOnApp = (LPTSTR)malloc(dwSize*sizeof(WCHAR));
			if (pKeyLockOnApp != NULL) {
				if (GetIniString(APP_SECTION, APP_KEYLOCK_ON_APP, pKeyLockOnApp, &dwSize, pIniBuf)) {
					;
				}
				else {
					free(pKeyLockOnApp);
					pKeyLockOnApp = NULL;
				}
			}
		}

		dwSize = 0;
		if (GetIniString(APP_SECTION, APP_KEYLOCK_ON_ARGS, NULL, &dwSize, pIniBuf)) {
			pKeyLockOnArgs = (LPTSTR)malloc(dwSize*sizeof(WCHAR));
			if (pKeyLockOnArgs != NULL) {
				if (GetIniString(APP_SECTION, APP_KEYLOCK_ON_ARGS, pKeyLockOnArgs, &dwSize, pIniBuf)) {
					;
				}
				else {
					free(pKeyLockOnArgs);
					pKeyLockOnArgs = NULL;
				}
			}
		}

		dwSize = 0;
		if (GetIniString(APP_SECTION, APP_KEYLOCK_OFF_APP, NULL, &dwSize, pIniBuf)) {
			pKeyLockOffApp = (LPTSTR)malloc(dwSize*sizeof(WCHAR));
			if (pKeyLockOffApp != NULL) {
				if (GetIniString(APP_SECTION, APP_KEYLOCK_OFF_APP, pKeyLockOffApp, &dwSize, pIniBuf)) {
					;
				}
				else {
					free(pKeyLockOffApp);
					pKeyLockOffApp = NULL;
				}
			}
		}

		dwSize = 0;
		if (GetIniString(APP_SECTION, APP_KEYLOCK_OFF_ARGS, NULL, &dwSize, pIniBuf)) {
			pKeyLockOffArgs = (LPTSTR)malloc(dwSize*sizeof(WCHAR));
			if (pKeyLockOffArgs != NULL) {
				if (GetIniString(APP_SECTION, APP_KEYLOCK_OFF_ARGS, pKeyLockOffArgs, &dwSize, pIniBuf)) {
					;
				}
				else {
					free(pKeyLockOffArgs);
					pKeyLockOffArgs = NULL;
				}
			}
		}

		pConfigData->pKeyLockOnApp = pKeyLockOnApp;
		pConfigData->pKeyLockOnArgs = pKeyLockOnArgs;
		pConfigData->pKeyLockOffApp = pKeyLockOffApp;
		pConfigData->pKeyLockOffArgs = pKeyLockOffArgs;

		bRet = TRUE;

		DiscardIniBuffer(pIniBuf);
		pIniBuf = NULL;
	}

	return bRet;
}

static
void
DiscardConfigData(
	CONFIGDATA*	pConfigData
)
{
	if (pConfigData != NULL) {
		if (pConfigData->pKeyLockOnApp != NULL) {
			free(pConfigData->pKeyLockOnApp);
			pConfigData->pKeyLockOnApp = NULL;
		}
		if (pConfigData->pKeyLockOnArgs != NULL) {
			free(pConfigData->pKeyLockOnArgs);
			pConfigData->pKeyLockOnArgs = NULL;
		}
		if (pConfigData->pKeyLockOffApp != NULL) {
			free(pConfigData->pKeyLockOffApp);
			pConfigData->pKeyLockOffApp = NULL;
		}
		if (pConfigData->pKeyLockOffArgs != NULL) {
			free(pConfigData->pKeyLockOffArgs);
			pConfigData->pKeyLockOffArgs = NULL;
		}
	}
}

static
BOOL
IsExecutableFile(
	LPTSTR	pFileName	// IN
)
{

#define	EXECUTABLE_SUFFIX	_T(".exe")

	BOOL	bRet;
	DWORD	dwSuffixLen = sizeof(EXECUTABLE_SUFFIX)/sizeof(TCHAR) - 1;

	bRet =  FALSE;

	if (pFileName != NULL) {
		DWORD	dwFilePathLen = _tcslen(pFileName);
		if (dwFilePathLen >= dwSuffixLen) {
			if (_tcsicmp(pFileName + dwFilePathLen - dwSuffixLen, EXECUTABLE_SUFFIX) == 0) {
				bRet = TRUE;
			}
		}
	}

	return bRet;
}

static
BOOL
ExecUserAction(
	LPTSTR	pAppPath,
	LPTSTR	pArgs
)
{
	BOOL	bRet;

	bRet = FALSE;	// default return value

	if (pAppPath == NULL) {
		return FALSE;
	}

	if (IsExecutableFile(pAppPath)) {

		PROCESS_INFORMATION pi;

		memset(&pi, 0, sizeof(pi));

		bRet = CreateProcess(pAppPath, pArgs, NULL, NULL, FALSE, 0, NULL, NULL, NULL, &pi);

		if (bRet) {
			CloseHandle(pi.hThread);
			CloseHandle(pi.hProcess);
		}

	}
	else {
		SHELLEXECUTEINFO	sei;

		ZeroMemory(&sei, sizeof(SHELLEXECUTEINFO));

		sei.cbSize = sizeof(sei);
		sei.lpVerb = _T("Open");
		sei.lpFile = pAppPath;

		bRet = ShellExecuteEx(&sei);
	}


	return bRet;
}


int
WINAPI
WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPTSTR    lpCmdLine,
	int       nCmdShow
)
{
	MSG msg;

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) {
		return FALSE;
	}

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}


ATOM
MyRegisterClass(
	HINSTANCE	hInstance,
	LPTSTR		szWindowClass
)
{
	WNDCLASS	wc;

    wc.style			= 0;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= 0;
    wc.hCursor			= 0;
    wc.hbrBackground	= 0;
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}


BOOL
InitInstance(
	HINSTANCE	hInstance,
	int			nCmdShow)
{
	HWND	hWnd = NULL;

	g_hInst = hInstance;


	hWnd = FindWindow(APP_CLASS_NAME, APP_TITLE_NAME);	
	if (hWnd) {
		int	iRet;

		iRet = MessageBox(NULL, _T("I܂?"), _T("KLChgExec"), MB_YESNO|MB_ICONQUESTION|MB_SETFOREGROUND);
		if (iRet == IDYES) {
			PostMessage(hWnd, WM_USER_APPEND_REQUEST, 0, 0);
		}
		return 0;
	} 

	MyRegisterClass(hInstance, APP_CLASS_NAME);
	
	hWnd = CreateWindow(APP_CLASS_NAME, APP_TITLE_NAME, 0, 0, 0, 0, 0, NULL, NULL, hInstance, NULL);
	if (hWnd == NULL) {	
		return FALSE;
	}

	return TRUE;
}


LRESULT
CALLBACK
WndProc(
	HWND	hWnd,
	UINT	message,
	WPARAM	wParam,
	LPARAM	lParam
)
{

	static CONFIGDATA	s_configData;
	static DWORD		s_dwStatus0 = 0;
	static HKEY			s_hShPhoneLib = NULL;
	static HREGNOTIFY	s_hRegNotify = NULL;

	switch (message) 
	{
		case WM_CREATE:
			{
				BOOL	bSucc;
				LONG	lResult;
				HRESULT	hr;
				TCHAR	szIniPath[MAX_PATH + 1];

				bSucc = FALSE;

				memset(szIniPath, 0, sizeof(szIniPath));
				if (GetIniPath(szIniPath)) {

					// init@C̓ǂݍ
					memset(&s_configData, 0, sizeof(s_configData));
					if (LoadConfigData(szIniPath, &s_configData)) {

						// RegistryNotifynAPȈ
						if (RNUtil_Initialize()) {

							// Status0̏l擾
							hr = RNUtil_RegistryGetDWORD(HKEY_CURRENT_USER, PHONESTATUS_REG_KEY, KEYLOCK_REG_VALUE_NAME, &s_dwStatus0);
							if (SUCCEEDED(hr)) {

								// PhoneStatus̃I[v
								lResult = RegOpenKeyEx(HKEY_CURRENT_USER, PHONESTATUS_REG_KEY, 0, NULL, &s_hShPhoneLib);
								if (lResult == ERROR_SUCCESS) {

									// Status0̕ύXĎJn
									hr = RNUtil_RegistryNotifyWindow(s_hShPhoneLib, NULL, KEYLOCK_REG_VALUE_NAME, hWnd, WM_USER_STATUS0_CHANGE, 0, NULL, &s_hRegNotify);
									if (SUCCEEDED(hr)) {
										bSucc = TRUE;
									}
									else {
										RegCloseKey(s_hShPhoneLib);
										s_hShPhoneLib = NULL;

										RNUtil_Terminate();

										DiscardConfigData(&s_configData);
									}
								}
								else {
									RNUtil_Terminate();
									DiscardConfigData(&s_configData);
								}
							}
							else {
								RNUtil_Terminate();
								DiscardConfigData(&s_configData);
							}
						}
						else {
							DiscardConfigData(&s_configData);
						}
					}
				}
				//

				if (!bSucc) {
					return -1;
				}
			}
			break;

		case WM_DESTROY:
			if (s_hRegNotify != NULL) {
				// Status0̕ύXĎI
				RNUtil_RegistryCloseNotification(s_hRegNotify);
				s_hRegNotify = NULL;
			}
			if (s_hShPhoneLib != NULL) {
				// PhoneStatus̃N[Y
				RegCloseKey(s_hShPhoneLib);
				s_hShPhoneLib = NULL;
			}
			
			// RegistryNotifynAPǏn
			RNUtil_Terminate();

			// ǂݍinit@Cf[^̔j
			DiscardConfigData(&s_configData);

			PostQuitMessage(0);
     		break;

		case WM_USER_STATUS0_CHANGE:
			{
				DWORD	dwNewStatus0 = wParam;

				if (s_dwStatus0 == 0) {
					if (dwNewStatus0 != 0) {
						// L[bNOFF -> L[bNON
						ExecUserAction(s_configData.pKeyLockOnApp, s_configData.pKeyLockOnArgs);
					}
				}
				else {
					if (dwNewStatus0 == 0) {
						// L[bNON -> L[bNOFF
						ExecUserAction(s_configData.pKeyLockOffApp, s_configData.pKeyLockOffArgs);
					}
				}

				s_dwStatus0 = dwNewStatus0;
			}
			break;

		case WM_USER_APPEND_REQUEST:
			DestroyWindow(hWnd);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }

   return 0;
}
