#include "stdafx.h"
#include "RegNotifyUtil.h"

/* Defines */
#define	AYGSHELL_DLL_NAME						_T("aygshell.dll")
#define REGISTRYNOTIFYWINDOW_FUNC_NAME			_T("RegistryNotifyWindow")
#define REGISTRYCLOSENOTIFICATION_FUNC_NAME		_T("RegistryCloseNotification")
#define REGISTRYGETDWORD_FUNC_NAME				_T("RegistryGetDWORD")

/* Typedefs */

typedef HRESULT (WINAPI * PFNREGISTRY_NOTIFY_WINDOW)(HKEY, LPCTSTR, LPCTSTR, HWND, UINT, DWORD, NOTIFICATIONCONDITION*, HREGNOTIFY*);
typedef HRESULT (WINAPI * PFNREGISTRY_CLOSE_NOTIFICATION)(HREGNOTIFY);
typedef HRESULT (WINAPI * PFNREGISTRY_GET_DWORD)(HKEY, LPCTSTR, LPCTSTR, DWORD*);


typedef struct {
	PFNREGISTRY_NOTIFY_WINDOW		pfnRegistryNotifyWindow;
	PFNREGISTRY_CLOSE_NOTIFICATION	pfnRegistryCloseNotification;
	PFNREGISTRY_GET_DWORD			pfnRegistryGetDWORD;
} REGNOTIFYFUNCENTRY;

/* Global Variables */
static	HMODULE				gs_hModule = NULL;
static	REGNOTIFYFUNCENTRY	gs_RNEntry;


static
BOOL
IsRNUtilInitialized()
{
	return (gs_hModule != NULL);
}


static
REGNOTIFYFUNCENTRY*
getEntry()
{
	return &gs_RNEntry;
}


BOOL
RNUtil_Initialize()
{
	BOOL		bRet;
	HMODULE		hModule = NULL;
	PFNREGISTRY_NOTIFY_WINDOW		pfnRegistryNotifyWindow			= NULL;
	PFNREGISTRY_CLOSE_NOTIFICATION	pfnRegistryCloseNotification	= NULL;
	PFNREGISTRY_GET_DWORD			pfnRegistryGetDWORD				= NULL;

	bRet = FALSE;	// default return value

	hModule = LoadLibrary(AYGSHELL_DLL_NAME);
	if (hModule != NULL) {
		pfnRegistryNotifyWindow			= (PFNREGISTRY_NOTIFY_WINDOW)		GetProcAddress(hModule, REGISTRYNOTIFYWINDOW_FUNC_NAME);
		pfnRegistryCloseNotification	= (PFNREGISTRY_CLOSE_NOTIFICATION)	GetProcAddress(hModule, REGISTRYCLOSENOTIFICATION_FUNC_NAME);
		pfnRegistryGetDWORD				= (PFNREGISTRY_GET_DWORD)			GetProcAddress(hModule, REGISTRYGETDWORD_FUNC_NAME);

		if (pfnRegistryNotifyWindow		!= NULL &&
			pfnRegistryCloseNotification != NULL &&
			pfnRegistryGetDWORD	!= NULL	) {

			gs_hModule = hModule;

			gs_RNEntry.pfnRegistryNotifyWindow		= pfnRegistryNotifyWindow;
			gs_RNEntry.pfnRegistryCloseNotification	= pfnRegistryCloseNotification;
			gs_RNEntry.pfnRegistryGetDWORD			= pfnRegistryGetDWORD;

			bRet = TRUE;
		}
	}

	return bRet;
}


BOOL
RNUtil_Terminate()
{
	BOOL	bRet;

	bRet = FALSE;

	if (gs_hModule != NULL) {
		FreeLibrary(gs_hModule);
		gs_hModule = NULL;

		bRet = TRUE;
	}

	return bRet;
}


HRESULT
RNUtil_RegistryNotifyWindow(
	HKEY					hKey,
	LPCTSTR					pszSubKey,
	LPCTSTR					pszValueName,
	HWND					hWnd,
	UINT					msg,
	DWORD					dwUserData,
	NOTIFICATIONCONDITION*	pCondition,
	HREGNOTIFY*				phNotify
)
{
	HRESULT	hr = E_FAIL;

	if (IsRNUtilInitialized()) {
		hr = (*(getEntry()->pfnRegistryNotifyWindow))(	hKey,
														pszSubKey,
														pszValueName,
														hWnd,
														msg,
														dwUserData,
														pCondition,
														phNotify
													);
	}

	return hr;
}


HRESULT
RNUtil_RegistryCloseNotification(
	HREGNOTIFY	hNotify
)
{
	HRESULT	hr = E_FAIL;

	if (IsRNUtilInitialized()) {
		hr = (*(getEntry()->pfnRegistryCloseNotification))(	hNotify );
	}

	return hr;
}

HRESULT
RNUtil_RegistryGetDWORD(
	HKEY	hKey,
	LPCTSTR	pszSubKey,
	LPCTSTR	pszValueName,
	DWORD*	pdwData
)
{
	HRESULT	hr = E_FAIL;

	if (IsRNUtilInitialized()) {
		hr = (*(getEntry()->pfnRegistryGetDWORD))(hKey, pszSubKey, pszValueName, pdwData); 
	}

	return hr;
}